from virgo_data_analyzer.measurement import Measurement
from virgo_data_analyzer.modelisation import Modelisation
from virgo_data_analyzer.modelisation.bench import Bench
from numpy import sin, cos, pi, sqrt
from gwpy.frequencyseries import FrequencySeries


class Projection:
    def __init__(
        self, measurement: Measurement, modelisation: Modelisation
    ):
        self.measurement = measurement
        self.modelisation = modelisation
        self.results: list[FrequencySeries] = []

        for bench in self.measurement.benches:
            modelisation_bench: None | Bench = None
            for i in range(len(self.modelisation.benches)):
                if bench.name == self.modelisation.benches[i].name:
                    modelisation_bench = self.modelisation.benches[i]
            if modelisation_bench is None:
                raise Exception(
                    "missing bench modelisation for {}".format(
                        bench.name
                    )
                )
            phase = 4 * pi / 1.064e-6
            sin_side = sin(
                bench.movement * phase
            ).asd(
                0.001
            )  # FIXME: To change (0.001 s: what are 10 points for the movement rate ?)
            cos_side = cos(bench.movement * phase).asd(0.001)
            self.results.append(
                sqrt(modelisation_bench.fsc)
                / phase
                * (
                    modelisation_bench.signal.kn * sin_side
                    + modelisation_bench.signal.kp * cos_side
                )
            )
